@@ -1,8 +1,20 @@
+#include <wx/defs.h>
+#include <libfilezilla/libfilezilla.hpp>
+#ifdef FZ_WINDOWS
+  #include <libfilezilla/private/windows.hpp>
+  #include <winsock2.h>
+  #include <ws2tcpip.h>
+#endif
 #include <filezilla.h>
 #include "engineprivate.h"
 #include "proxy.h"
 #include "ControlSocket.h"
-#include <wx/sckaddr.h>
+
+#include <libfilezilla/iputils.hpp>
+
+#ifndef FZ_WINDOWS
+#include <netdb.h>
+#endif
 
 enum handshake_state
 {
@@ -17,8 +29,8 @@ enum handshake_state
 	socks4_handshake
 };
 
-CProxySocket::CProxySocket(CEventHandler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner)
-	: CEventHandler(pOwner->event_loop_)
+CProxySocket::CProxySocket(fz::event_handler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner)
+	: fz::event_handler(pOwner->event_loop_)
 	, CBackend(pEvtHandler)
 	, m_pSocket(pSocket)
 	, m_pOwner(pOwner)
@@ -28,7 +40,7 @@ CProxySocket::CProxySocket(CEventHandler* pEvtHandler, CSocket* pSocket, CContro
 
 CProxySocket::~CProxySocket()
 {
-	RemoveHandler();
+	remove_handler();
 
 	if (m_pSocket)
 		m_pSocket->SetEventHandler(0);
@@ -36,7 +48,8 @@ CProxySocket::~CProxySocket()
 	delete [] m_pRecvBuffer;
 }
 
-static wxString base64encode(const wxString& str)
+namespace {
+wxString base64encode(const wxString& str)
 {
 	// Code shamelessly taken from wxWidgets and adopted to encode UTF-8 strings.
 	// wxWidget's http class encodes string from arbitrary encoding into base64,
@@ -59,7 +72,8 @@ static wxString base64encode(const wxString& str)
 		buf << wxString::Format(wxT("%c"), base64[(from[0] >> 2) & 0x3f]);
 		if (len == 1) {
 			buf << wxString::Format(wxT("%c="), base64[(from[0] << 4) & 0x30]);
-		} else {
+		}
+		else {
 			buf << wxString::Format(wxT("%c%c"), base64[((from[0] << 4) & 0x30) | ((from[1] >> 4) & 0xf)], base64[(from[1] << 2) & 0x3c]);
 		}
 		buf << wxString::Format(wxT("="));
@@ -67,6 +81,21 @@ static wxString base64encode(const wxString& str)
 
 	return buf;
 }
+}
+
+wxString CProxySocket::Name(ProxyType t)
+{
+	switch (t) {
+	case HTTP:
+		return _T("HTTP");
+	case SOCKS4:
+		return _T("SOCKS4");
+	case SOCKS5:
+		return _T("SOCKS5");
+	default:
+		return _("unknown");
+	}
+}
 
 int CProxySocket::Handshake(CProxySocket::ProxyType type, const wxString& host, unsigned int port, const wxString& user, const wxString& pass)
 {
@@ -120,18 +149,29 @@ int CProxySocket::Handshake(CProxySocket::ProxyType type, const wxString& host,
 	}
 	else if (type == SOCKS4) {
 		wxString ip = m_host;
-		if (!GetIPV6LongForm(m_host).empty()) {
+		if (!fz::get_ipv6_long_form(m_host.ToStdWstring()).empty()) {
 			m_pOwner->LogMessage(MessageType::Error, _("IPv6 addresses are not supported with SOCKS4 proxy"));
 			return EINVAL;
 		}
 
-		if (!IsIpAddress(m_host)) {
-			wxIPV4address address;
-			if (!address.Hostname(host)) {
+		if (fz::get_address_type(m_host.ToStdWstring()) == fz::address_type::unknown) {
+			addrinfo hints{};
+			hints.ai_family = AF_INET;
+			hints.ai_socktype = SOCK_STREAM;
+
+			addrinfo * result{};
+			int res = getaddrinfo(host.ToStdString().c_str(), 0, &hints, &result);
+			if (!res && result) {
+				if (result->ai_family == AF_INET) {
+					ip = CSocket::AddressToString(result->ai_addr, result->ai_addrlen, false);
+				}
+				freeaddrinfo(result);
+			}
+
+			if (ip.empty()) {
 				m_pOwner->LogMessage(MessageType::Error, _("Cannot resolve hostname to IPv4 address for use with SOCKS4 proxy."));
 				return EINVAL;
 			}
-			ip = address.IPAddress();
 		}
 
 		m_pOwner->LogMessage(MessageType::Status, _("SOCKS4 proxy will connect to: %s"), ip);
@@ -185,9 +225,9 @@ int CProxySocket::Handshake(CProxySocket::ProxyType type, const wxString& host,
 	return EINPROGRESS;
 }
 
-void CProxySocket::operator()(CEventBase const& ev)
+void CProxySocket::operator()(fz::event_base const& ev)
 {
-	Dispatch<CSocketEvent, CHostAddressEvent>(ev, this,
+	fz::dispatch<CSocketEvent, CHostAddressEvent>(ev, this,
 		&CProxySocket::OnSocketEvent,
 		&CProxySocket::OnHostAddress);
 }
@@ -203,7 +243,7 @@ void CProxySocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int err
 		if (error) {
 			if (m_proxyState == handshake)
 				m_proxyState = noconn;
-			m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::connection, error);
+			m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::connection, error);
 		}
 		else {
 			m_pOwner->LogMessage(MessageType::Status, _("Connection with proxy established, performing handshake..."));
@@ -258,7 +298,7 @@ void CProxySocket::OnReceive()
 				if (read == -1) {
 					if (error != EAGAIN) {
 						m_proxyState = noconn;
-						m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, error);
+						m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, error);
 					}
 					else
 						m_can_read = false;
@@ -266,13 +306,13 @@ void CProxySocket::OnReceive()
 				}
 				if (!read) {
 					m_proxyState = noconn;
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
 				if (m_pSendBuffer) {
 					m_proxyState = noconn;
 					m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Incoming data before request fully sent"));
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
 
@@ -284,7 +324,7 @@ void CProxySocket::OnReceive()
 						if (m_recvBufferPos + read + 1 == m_recvBufferLen) {
 							m_proxyState = noconn;
 							m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Incoming header too large"));
-							m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ENOMEM);
+							m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ENOMEM);
 							return;
 						}
 						do_read = read;
@@ -296,7 +336,7 @@ void CProxySocket::OnReceive()
 					if (read != do_read) {
 						m_proxyState = noconn;
 						m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Could not read what got peeked"));
-						m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+						m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 						return;
 					}
 					m_recvBufferPos += read;
@@ -313,12 +353,12 @@ void CProxySocket::OnReceive()
 
 			if (reply.Left(10) != _T("HTTP/1.1 2") && reply.Left(10) != _T("HTTP/1.0 2")) {
 				m_proxyState = noconn;
-				m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNRESET);
+				m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNRESET);
 				return;
 			}
 
 			m_proxyState = conn;
-			m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::connection, 0);
+			m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::connection, 0);
 			return;
 		}
 	case socks4_handshake:
@@ -328,7 +368,7 @@ void CProxySocket::OnReceive()
 			if (read == -1) {
 				if (read_error != EAGAIN) {
 					m_proxyState = noconn;
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, read_error);
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, read_error);
 				}
 				else
 					m_can_read = false;
@@ -337,7 +377,7 @@ void CProxySocket::OnReceive()
 
 			if (!read) {
 				m_proxyState = noconn;
-				m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+				m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 				return;
 			}
 			m_recvBufferPos += read;
@@ -366,11 +406,11 @@ void CProxySocket::OnReceive()
 				}
 				m_pOwner->LogMessage(MessageType::Error, _("Proxy request failed: %s"), error);
 				m_proxyState = noconn;
-				m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+				m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 				return;
 			}
 			m_proxyState = conn;
-			m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::connection, 0);
+			m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::connection, 0);
 		}
 		return;
 	case socks5_method:
@@ -386,7 +426,7 @@ void CProxySocket::OnReceive()
 			if (read == -1) {
 				if (error != EAGAIN) {
 					m_proxyState = noconn;
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, error);
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, error);
 				}
 				else
 					m_can_read = false;
@@ -394,7 +434,7 @@ void CProxySocket::OnReceive()
 			}
 			if (!read) {
 				m_proxyState = noconn;
-				m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+				m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 				return;
 			}
 			m_recvBufferPos += read;
@@ -409,17 +449,17 @@ void CProxySocket::OnReceive()
 			switch (m_handshakeState) {
 			default:
 				if (m_pRecvBuffer[0] != 5) {
-					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Unknown SOCKS protocol version: %d"), (int)m_pRecvBuffer[0]);
+					m_pOwner->LogMessage(MessageType::Error, _("Unknown SOCKS protocol version: %d"), (int)m_pRecvBuffer[0]);
 					m_proxyState = noconn;
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
 				break;
 			case socks5_auth:
 				if (m_pRecvBuffer[0] != 1) {
-					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Unknown protocol version of SOCKS Username/Password Authentication subnegotiation: %d"), (int)m_pRecvBuffer[0]);
+					m_pOwner->LogMessage(MessageType::Error, _("Unknown protocol version of SOCKS Username/Password Authentication subnegotiation: %d"), (int)m_pRecvBuffer[0]);
 					m_proxyState = noconn;
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
 				break;
@@ -441,9 +481,9 @@ void CProxySocket::OnReceive()
 						m_handshakeState = socks5_auth;
 						break;
 					default:
-						m_pOwner->LogMessage(MessageType::Debug_Warning, _("No supported SOCKS5 auth method"));
+						m_pOwner->LogMessage(MessageType::Error, _("No supported SOCKS5 auth method"));
 						m_proxyState = noconn;
-						m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+						m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 						return;
 					}
 				}
@@ -451,9 +491,9 @@ void CProxySocket::OnReceive()
 			case socks5_auth:
 				if (m_pRecvBuffer[1] != 0)
 				{
-					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Proxy authentication failed"));
+					m_pOwner->LogMessage(MessageType::Error, _("Proxy authentication failed"));
 					m_proxyState = noconn;
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
 				m_handshakeState = socks5_request;
@@ -492,9 +532,9 @@ void CProxySocket::OnReceive()
 						break;
 					}
 
-					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Proxy request failed: %s"), errorMsg);
+					m_pOwner->LogMessage(MessageType::Error, _("Proxy request failed. Reply from proxy: %s"), errorMsg);
 					m_proxyState = noconn;
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
 				m_handshakeState = socks5_request_addrtype;
@@ -516,9 +556,9 @@ void CProxySocket::OnReceive()
 					m_recvBufferLen = 17;
 					break;
 				default:
-					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Proxy request failed: Unknown address type in CONNECT reply"));
+					m_pOwner->LogMessage(MessageType::Error, _("Proxy request failed: Unknown address type in CONNECT reply"));
 					m_proxyState = noconn;
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
 				m_handshakeState = socks5_request_address;
@@ -527,7 +567,7 @@ void CProxySocket::OnReceive()
 				{
 					// We're done
 					m_proxyState = conn;
-					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::connection, 0);
+					m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::connection, 0);
 					return;
 				}
 			default:
@@ -565,37 +605,34 @@ void CProxySocket::OnReceive()
 					m_pSendBuffer[1] = 1; // CONNECT
 					m_pSendBuffer[2] = 0; // Reserved
 
-					if (IsIpAddress(m_host)) {
-						// Quite ugly
-						wxString ipv6 = GetIPV6LongForm(m_host);
-						if (!ipv6.empty()) {
-							ipv6.Replace(_T(":"), _T(""));
-							addrlen = 16;
-							for (int i = 0; i < 16; i++)
-								m_pSendBuffer[4 + i] = (DigitHexToDecNum(ipv6[i * 2]) << 4) + DigitHexToDecNum(ipv6[i * 2 + 1]);
-
-							m_pSendBuffer[3] = 4; // IPv6
+					auto type = fz::get_address_type(m_host.ToStdWstring());
+					if (type == fz::address_type::ipv6) {
+						auto ipv6 = fz::get_ipv6_long_form(m_host.ToStdWstring());
+						addrlen = 16;
+						for (int i = 0; i < 16; ++i) {
+							m_pSendBuffer[4 + i] = (fz::hex_char_to_int(ipv6[i * 2 + i / 2]) << 4) + fz::hex_char_to_int(ipv6[i * 2 + 1 + i / 2]);
 						}
-						else {
-							unsigned char *buf = (unsigned char*)m_pSendBuffer + 4;
-							int i = 0;
-							memset(buf, 0, 4);
-							for (const wxChar* p = m_host.c_str(); *p && i < 4; p++)
+
+						m_pSendBuffer[3] = 4; // IPv6
+					}
+					else if (type == fz::address_type::ipv4) {
+						unsigned char *buf = (unsigned char*)m_pSendBuffer + 4;
+						int i = 0;
+						memset(buf, 0, 4);
+						for (const wxChar* p = m_host.c_str(); *p && i < 4; ++p) {
+							const wxChar& c = *p;
+							if (c == '.')
 							{
-								const wxChar& c = *p;
-								if (c == '.')
-								{
-									i++;
-									continue;
-								}
-								buf[i] *= 10;
-								buf[i] += c - '0';
+								i++;
+								continue;
 							}
-
-							addrlen = 4;
-
-							m_pSendBuffer[3] = 1; // IPv4
+							buf[i] *= 10;
+							buf[i] += c - '0';
 						}
+
+						addrlen = 4;
+
+						m_pSendBuffer[3] = 1; // IPv4
 					}
 					else {
 						m_pSendBuffer[3] = 3; // Domain name
@@ -627,7 +664,7 @@ void CProxySocket::OnReceive()
 	default:
 		m_proxyState = noconn;
 		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Unhandled handshake state %d"), m_handshakeState);
-		m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
+		m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 		return;
 	}
 }
@@ -644,7 +681,7 @@ void CProxySocket::OnSend()
 		if (written == -1) {
 			if (error != EAGAIN) {
 				m_proxyState = noconn;
-				m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, error);
+				m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, error);
 			}
 			else
 				m_can_write = false;
